"""
    Copyright 2023 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""

from typing import Optional

import pkg_resources
from pkg_resources import DistributionNotFound

from packaging import version

CORE_VERSION: Optional[version.Version]
"""
Version of the inmanta-core package. None if it is not installed.
"""

try:
    CORE_VERSION = version.Version(
        pkg_resources.get_distribution("inmanta-core").version
    )
except DistributionNotFound:
    CORE_VERSION = None

# Setting a project-wide pip index is only supported for iso7+
SUPPORTS_PROJECT_PIP_INDEX: bool = (
    CORE_VERSION is not None and CORE_VERSION >= version.Version("9.0.0.dev")
)

SUPPORTS_MODULES_V2: bool = (
    CORE_VERSION is not None and CORE_VERSION >= version.Version("6.dev")
)



"""
    Copyright 2019 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""
# Note: These tests only function when the pytest output is not modified by plugins such as pytest-sugar!


def test_basic_example(testdir):
    """Make sure that our plugin works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_resource_run.py")

    result.assert_outcomes(passed=1)


def test_dryrun_example(testdir):
    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_dryrun.py")

    result.assert_outcomes(passed=1)


def test_run_sync(testdir):
    """Make sure that the run_sync mock works."""

    testdir.copy_example("testsync")

    result = testdir.runpytest("tests/test_stuff.py")

    result.assert_outcomes(passed=1)


def test_run_reflection(testdir):
    """Make sure that the run_sync mock works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_reflection.py")

    result.assert_outcomes(passed=1)


def test_run_capture(testdir):
    """Make sure that the run_sync mock works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_capture.py")

    result.assert_outcomes(passed=1)


def test_fixture_reset(testdir):
    """Make sure that the run_sync mock works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_reset.py")

    result.assert_outcomes(passed=2)


def test_badlog(testdir):
    """Make sure that the run_sync mock works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_BadLog.py")

    result.assert_outcomes(xfailed=1)


def test_resource_bad_id_attribute(testdir):
    """Make sure a deprecation warning is shown if an id_attribute called 'id' is found."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_resource_bad_id_attribute.py")

    result.assert_outcomes(passed=1)
    assert (
        "In one of the next major releases of inmanta-core it will not be possible anymore "
        "to use an id_attribute called id for testmodule::ResourceBadIdAttribute"
    ) in result.stdout.str()


def test_release_mode_validation(testdir):
    """Set invalid release mode"""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_resource_run.py", "--install_mode", "other")
    assert (
        "error: argument --install_mode: invalid choice: 'other' (choose from"
        in "\n".join(result.errlines)
    )


def test_multiple_repo_paths_option(testdir):
    testdir.copy_example("testmodule")

    result = testdir.runpytest(
        "tests/test_multiple_repo_paths.py",
        "--module_repo",
        "https://github.com/inmanta2/ https://github.com/inmanta/",
    )
    result.assert_outcomes(passed=1)


def test_multiple_repo_paths_multiple_options(testdir):
    testdir.copy_example("testmodule")

    result = testdir.runpytest(
        "tests/test_multiple_repo_paths.py",
        "--module_repo",
        "https://github.com/inmanta2/",
        "--module_repo",
        "https://github.com/inmanta/",
    )
    result.assert_outcomes(passed=1)


def test_multiple_repo_paths_env(testdir, monkeypatch):
    monkeypatch.setenv(
        "INMANTA_MODULE_REPO",
        "https://github.com/inmanta2/ https://github.com/inmanta/",
    )
    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_multiple_repo_paths.py")
    result.assert_outcomes(passed=1)


def test_import(testdir):
    """Make sure that importing functions works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_import.py")

    result.assert_outcomes(passed=5)


def test_project_no_plugins(testdir):
    """Make sure that using the project_no_plugins shows a warning."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_project_no_plugins.py")

    result.assert_outcomes(passed=1)
    assert (
        "DeprecationWarning: The project_no_plugins fixture is deprecated"
        " in favor of the INMANTA_NO_LOAD_PLUGINS environment variable."
    ) in result.stdout.str()


def test_state(testdir):
    """Make sure that importing functions works."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_state.py")

    result.assert_outcomes(passed=2)


def test_cwd(testdir):
    """Ensure that the project fixture resets the cwd after each test case."""

    testdir.copy_example("testmodule")

    result = testdir.runpytest("tests/test_cwd.py")

    result.assert_outcomes(passed=2)



"""
    Copyright 2021 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""
# Note: These tests only function when the pytest output is not modified by plugins such as pytest-sugar!
import logging
import os
import tempfile

import pytest

import pytest_inmanta.plugin
import utils
from inmanta import env
from pytest_inmanta.core import SUPPORTS_PROJECT_PIP_INDEX
from pytest_inmanta.parameters import pip_index_url


def test_transitive_v2_dependencies(
    examples_v2_package_index, pytestconfig, testdir, caplog
):
    # set working directory to allow in-place with all example modules
    pytest_inmanta.plugin.CURDIR = str(
        pytestconfig.rootpath / "examples" / "test_dependencies_head"
    )

    testdir.copy_example("test_dependencies_head")

    with caplog.at_level(logging.WARNING):
        with tempfile.TemporaryDirectory() as venv_dir:
            # set up environment
            venv: env.VirtualEnv = env.VirtualEnv(env_path=venv_dir)
            try:
                venv.use_virtual_env()

                # run tests
                result = testdir.runpytest_inprocess(
                    "tests/test_basics.py",
                    "--use-module-in-place",
                    # add pip index containing examples packages as module repo
                    "--pip-index-url",
                    f"{examples_v2_package_index}",
                    # include configured pip index for inmanta-module-std
                    "--pip-index-url",
                    f'{os.environ.get("PIP_INDEX_URL", "https://pypi.org/simple")}',
                )
                result.assert_outcomes(passed=1)
            finally:
                utils.unload_modules_for_path(venv.site_packages_dir)

        if not SUPPORTS_PROJECT_PIP_INDEX:
            warning_msg: str = (
                "Setting a project-wide pip index is not supported on this version of inmanta-core. "
                "The provided index will be used as a v2 package source"
            )
            assert warning_msg in caplog.text


@pytest.mark.parametrize(
    "no_strict_deps_check, error_msg",
    [
        (True, "CompilerException"),
        (False, "ConflictingRequirements"),
    ],
)
def test_conflicing_dependencies(
    examples_v2_package_index, pytestconfig, testdir, no_strict_deps_check, error_msg
):
    """
    when using the pytest-inmanta without specifying the --no-strict-deps-check, the constraints
    of the installed modules/packages are verified and if a conflict is detected a ConflictingRequirement
    error is raised.
    when using pytest-inmanta with --no-strict-deps-check option,
    the legacy check on the constraints is done. If the installed modules are not compatible
    a CompilerException is raised. In the used example for this test,
    test_conflict_dependencies(v1 module) requires inmanta-module-testmodulev2conflict1 and
    inmanta-module-testmodulev2conflict2. The later two are incompatible as one requires lorem 0.0.1
    and the other one 0.1.1.
    """
    # set working directory to allow in-place with all example modules
    pytest_inmanta.plugin.CURDIR = str(
        pytestconfig.rootpath / "examples" / "test_conflict_dependencies"
    )
    testdir.copy_example("test_conflict_dependencies")

    with tempfile.TemporaryDirectory() as venv_dir:
        # set up environment
        venv: env.VirtualEnv = env.VirtualEnv(env_path=venv_dir)
        try:
            venv.use_virtual_env()

            # run tests
            result = testdir.runpytest_inprocess(
                "tests/test_basics.py",
                *(["--no-strict-deps-check"] if no_strict_deps_check else []),
                "--use-module-in-place",
                # add pip index containing examples packages as module repo
                "--pip-index-url",
                f"{examples_v2_package_index}",
                # include configured pip index for inmanta-module-std and lorem
                "--pip-index-url",
                f'{os.environ.get("PIP_INDEX_URL", "https://pypi.org/simple")}',
            )
            result.assert_outcomes(errors=1)
            assert error_msg in "\n".join(result.outlines)
        finally:
            utils.unload_modules_for_path(venv.site_packages_dir)


def test_transitive_v2_dependencies_legacy_warning(
    examples_v2_package_index, pytestconfig, testdir, caplog
):
    # set working directory to allow in-place with all example modules
    pytest_inmanta.plugin.CURDIR = str(
        pytestconfig.rootpath / "examples" / "test_dependencies_head"
    )

    testdir.copy_example("test_dependencies_head")

    with caplog.at_level(logging.WARNING):
        with tempfile.TemporaryDirectory() as venv_dir:
            # set up environment
            venv: env.VirtualEnv = env.VirtualEnv(env_path=venv_dir)
            try:
                venv.use_virtual_env()

                # run tests
                result = testdir.runpytest_inprocess(
                    "tests/test_basics.py",
                    "--use-module-in-place",
                    # add pip index containing examples packages as module repo
                    "--module_repo",
                    f"package:{examples_v2_package_index}",
                    # include configured pip index for inmanta-module-std
                    "--module_repo",
                    "package:"
                    + os.environ.get(
                        "PIP_INDEX_URL", "package:https://pypi.org/simple"
                    ),
                )
                result.assert_outcomes(passed=1)
            finally:
                utils.unload_modules_for_path(venv.site_packages_dir)

        if SUPPORTS_PROJECT_PIP_INDEX:
            warning_msg: str = (
                "Setting a package source through the --module-repo <index_url> cli option with type `package` "
                "is now deprecated and will raise a warning during compilation."
                " Use the --pip-index-url <index_url> pytest option instead or set"
                f" the {pip_index_url.environment_variable} environment variable to address these warnings. "
            )
            assert warning_msg in caplog.text



"""
    Copyright 2020 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""
import os
import shutil
import uuid

import pytest_inmanta


def test_module_in_place(testdir):
    """Make sure the run in place option works"""

    # copy_example copies what is IN the given directory, not the directory itself...
    testdir.copy_example("testmodule")
    # Moving module to make sure we can run in place,
    # by making sure the module name in module.yml is in it's parent path
    os.mkdir("testmodule")
    shutil.move("model", "testmodule/model")
    shutil.move("module.yml", "testmodule/module.yml")
    shutil.move("plugins", "testmodule/plugins")
    shutil.move("tests", "testmodule/tests")

    os.chdir("testmodule")
    path = os.getcwd()
    assert not os.path.exists(os.path.join(path, "testfile"))
    pytest_inmanta.plugin.CURDIR = path

    result = testdir.runpytest("tests/test_location.py", "--use-module-in-place")

    result.assert_outcomes(passed=1)

    assert os.path.exists(os.path.join(path, "testfile"))


def test_not_existing_venv_option(testdir, tmpdir):
    testdir.copy_example("testmodule")
    venv_path = os.path.join(tmpdir, str(uuid.uuid4()))

    result = testdir.runpytest("tests/test_resource_run.py", "--venv", venv_path)

    result.assert_outcomes(errors=1)
    assert f"Specified venv {venv_path} does not exist" in "\n".join(result.outlines)



"""
    Copyright 2022 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""
import logging
import os
import uuid
from abc import abstractmethod
from collections import defaultdict
from enum import Enum
from typing import Container, Dict, Generic, List, Optional, Set, TypeVar, Union

try:
    """
    Those classes are only used in type annotation, but the import doesn't work
    in python 3.6.  So we simply catch the error and ignore it.
    """
    from pytest import Config, OptionGroup, Parser
except ImportError:
    pass

LOGGER = logging.getLogger(__name__)


ParameterType = TypeVar("ParameterType")
"""
The parameter type is a TypeVar which specify which type a specific TestParameter
instance or class will resolve.
"""


class ParameterNotSetException(ValueError):
    """
    This exception is raised when a parameter is accessed but no value has
    been set by the user.
    """

    def __init__(self, parameter: "TestParameter") -> None:
        super().__init__(
            f"Couldn't resolve a test parameter.  "
            f"You can set it using {parameter.argument} argument or "
            f"{parameter.environment_variable} environment variable."
        )
        self.parameter = parameter


class TestParameterRegistry:
    """
    Singleton class that keeps information about registered test parameters
    """

    __test_parameters: Dict[str, "TestParameter"] = dict()
    __test_parameter_groups: Dict[Optional[str], Set["TestParameter"]] = defaultdict(
        set
    )
    __parser: Optional["Parser"] = None

    @staticmethod
    def add_option(
        parser: "Parser", group_name: Optional[str], test_parameter: "TestParameter"
    ) -> None:
        """
        This static method adds the option defined in test_parameter to the parser provided in argument.
        If group_name is not None, the option will be added to the group named :param group_name:.

        :param parser: The parser to register the option in
        :param group_name: The name of the group the option is a part of
        :param test_parameter: The test parameter holding all the option's information
        """
        group: Union["Parser", "OptionGroup"]
        if group_name is None:
            group = parser
        else:
            group = parser.getgroup(group_name)

        kwargs: Dict[str, object] = dict(
            action=test_parameter.action,
            help=test_parameter.help,
            # We overwrite the default here, to ensure that even boolean options don't default to the opposite of
            # the store action.  If we don't do this, config.getoption will always return a value, either True or
            # False depending on the action and whether the flag is set or not, this makes it impossible to use
            # environment variables for the option.
            default=None,
        )
        if test_parameter.choices is not None:
            kwargs["choices"] = test_parameter.choices

        group.addoption(test_parameter.argument, **kwargs)

    @classmethod
    def register(
        cls,
        key: Optional[str],
        test_parameter: "TestParameter",
        group: Optional[str] = None,
    ) -> None:
        """
        Register a parameter, you should not call this method your self.
        This method is called by the constructor of TestParameter.
        """
        if key is None:
            key = str(uuid.uuid4())

        cls.__test_parameters[key] = test_parameter
        cls.__test_parameter_groups[group].add(test_parameter)

        if cls.__parser is not None:
            # Pytest has already loaded this plugin, we need to add the option now
            TestParameterRegistry.add_option(cls.__parser, group, test_parameter)

    @classmethod
    def test_parameters(cls) -> List["TestParameter"]:
        """
        Get all the registered parameters
        """
        return sorted(cls.__test_parameters.values(), key=lambda param: param.argument)

    @classmethod
    def test_parameter_groups(cls) -> Dict[Optional[str], List["TestParameter"]]:
        """
        Get all the registered parameters, grouped by group name.  The output is a dict holding
        for each group name (key) the list of all parameters (value).  The parameters which are
        not part of a group are grouped in a list at key None.
        """
        return {
            group: sorted(parameters, key=lambda param: param.argument)
            for group, parameters in cls.__test_parameter_groups.items()
        }

    @classmethod
    def test_parameter(cls, key: str) -> "TestParameter":
        """
        Get the parameter that was created with key :param key:, if it is not found, raise a KeyError
        """
        return cls.__test_parameters[key]

    @classmethod
    def pytest_addoption(cls, parser: "Parser") -> None:
        """
        This method should be called once (and only once) in pytest_inmanta.plugin.pytest_addoption
        It will register the parser for later use and setup all the options that have already been
        registered.
        """
        if cls.__parser == parser:
            raise RuntimeError("Options can not be registered more than once")

        # Saving the parser for late option registration
        cls.__parser = parser

        # We setup all the options that are already registered
        for group_name, parameters in cls.test_parameter_groups().items():
            for param in parameters:
                TestParameterRegistry.add_option(parser, group_name, param)


class ValueSetBy(Enum):
    """
    This class is used to record how the value was provided for a test parameter.
    """

    DEFAULT_VALUE: str = "DEFAULT_VALUE"
    CLI: str = "CLI"
    ENV_VARIABLE: str = "ENV_VARIABLE"


class TestParameter(Generic[ParameterType]):
    """
    This class represents a parameter that can be passed to the tests, either via a pytest
    argument, or via an environment variable.
    """

    def __init__(
        self,
        argument: str,
        environment_variable: str,
        usage: str,
        *,
        default: Optional[ParameterType] = None,
        key: Optional[str] = None,
        group: Optional[str] = None,
        legacy: Optional["TestParameter[ParameterType]"] = None,
    ) -> None:
        """
        :param argument: This is the argument that can be passed to the pytest command.
        :param environment_variable: This is the name of the environment variable in which
            the value can be stored.
        :param usage: This is a small description of what the parameter value will be used for.
        :param default: This is the default value to provide if the parameter is resolved but
            hasn't been set.
        :param key: Optionally, a key can be set, its sole purpose is to allow the creator of
            the option to access it directly from the parameter registry, thanks to this key,
            using TestParameterRegistry.test_parameter(<the-key>)
        :param group: A group in which the option should be added.  If None is provided, the
            option isn't part of any group.
        :param legacy: An optional legacy parameter, that this one replaces, but will be removed
            in future version of the product.  When resolving a value, we first check this
            parameter, and if it is not set, we check the legacy one and raise a warning about
            its deprecation.
        """
        self.argument = argument
        self.environment_variable = environment_variable
        self.usage = usage
        self.default = default
        self.legacy = legacy
        # Track how the value was set when it is being resolved:
        self._value_set_using: Optional[str] = None

        TestParameterRegistry.register(key, self, group)

    @property
    def help(self) -> str:
        """
        Build up a help message, based on the usage, default value and environment variable name.
        """
        additional_messages = [f"overrides {self.environment_variable}"]
        if self.default is not None:
            additional_messages.append(f"defaults to {self.default}")

        return self.usage + f" ({', '.join(additional_messages)})"

    @property
    def action(self) -> str:
        """
        The argparse action for this option
        https://docs.python.org/3/library/argparse.html#action
        """
        return "store"

    @property
    def choices(self) -> Optional[Container[str]]:
        """
        The argparse choices for this option
        https://docs.python.org/3/library/argparse.html#choices
        """
        return None

    @abstractmethod
    def validate(self, raw_value: object) -> ParameterType:
        """
        This method is called when any value is received from parameters or
        env variables.  It is given in the raw_value argument a string conversion
        of the received value.  It is up to the class extending this one to convert
        it to whatever value it wants.
        """

    def resolve(self, config: "Config") -> ParameterType:
        """
        Resolve the test parameter.
        First, we try to get it from the provided options.
        Second, we try to get it from environment variables.
        Then, if there is a default, we use it.
        Finally, if none of the above worked, we raise a ParameterNotSetException.
        """
        option = config.getoption(self.argument, default=None)
        if option is not None:
            # A value is set, and it is not the default one
            return self.validate(option)

        env_var = os.getenv(self.environment_variable)
        if env_var is not None:
            # A value is set
            return self.validate(env_var)

        if self.legacy is not None:
            # If we have a legacy option, we check if it is set
            try:
                val = self.legacy.resolve(config)
                LOGGER.warning(
                    f"The usage of {self.legacy.argument} is deprecated, "
                    f"use {self.argument} instead"
                )
                return val
            except ParameterNotSetException:
                pass

        if self.default is not None:
            return self.default

        raise ParameterNotSetException(self)



"""
    Copyright 2021 Inmanta

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

        http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.

    Contact: code@inmanta.com
"""
import logging

# Note: These tests only function when the pytest output is not modified by plugins such as pytest-sugar!
from typing import Iterator

import pytest

import utils
from inmanta import env

# be careful not to import any core>=6 objects directly
from pytest_inmanta.core import SUPPORTS_MODULES_V2

if not SUPPORTS_MODULES_V2:
    pytest.skip(
        "Skipping modules v2 tests for inmanta-core<6 (pre modules v2).",
        allow_module_level=True,
    )


@pytest.fixture(scope="function")
def testmodulev2_editable_install() -> bool:
    raise NotImplementedError(
        "Tests using the testmodulev2 should have a marker specifying whether the module "
        "should be installed in editable mode or not."
    )


@pytest.fixture(scope="function")
def testmodulev2_venv(
    testdir: pytest.Testdir,
    pytestconfig: pytest.Config,
    testmodulev2_editable_install: bool,
) -> Iterator[env.VirtualEnv]:
    """
    Yields a Python environment with testmodulev2 installed in it.
    """
    module_dir = testdir.copy_example("testmodulev2")
    with utils.module_v2_venv(
        module_dir, editable_install=testmodulev2_editable_install
    ) as venv:
        yield venv


@pytest.fixture(scope="function")
def testmodulev2_venv_active(
    deactive_venv: None,
    testmodulev2_venv: env.VirtualEnv,
) -> Iterator[env.VirtualEnv]:
    """
    Activates a Python environment with testmodulev2 installed in it for the currently running process.
    """
    with utils.activate_venv(testmodulev2_venv) as venv:
        yield venv


@pytest.mark.parametrize("testmodulev2_editable_install", [False, True])
def test_basic_example(
    testdir: pytest.Testdir,
    caplog: pytest.LogCaptureFixture,
    testmodulev2_venv_active: env.VirtualEnv,
    testmodulev2_editable_install: bool,
) -> None:
    """
    Make sure that our plugin works for v2 modules.
    """
    caplog.clear()
    with caplog.at_level(logging.WARNING):
        result = testdir.runpytest_inprocess("tests/test_basics.py")
        result.assert_outcomes(passed=1)

        if not testmodulev2_editable_install:
            # The testmodulev2_venv_active fixture does not install the module in editable mode. For pytest-inmanta tests
            # this is fine but for module testing this is likely a mistake. Verify that the plugin raises an appropriate
            # warning.
            assert (
                "The module being tested is not installed in editable mode."
                " As a result the tests will not pick up any changes to the local source files."
                " To install it in editable mode, run `inmanta module install -e .`."
                in caplog.messages
            )


def test_basic_example_no_install(testdir: pytest.Testdir) -> None:
    """
    Make sure that the plugin reports an informative error if the module under test is not installed.
    """
    testdir.copy_example("testmodulev2")

    result = testdir.runpytest_inprocess("tests/test_basics.py::test_compile")

    result.assert_outcomes(errors=1)
    result.stdout.re_match_lines(
        [
            r".*Exception: The module being tested is not installed in the current Python environment\."
            r" Please install it with `inmanta module install -e \.` before running the tests\..*"
        ]
    )


@pytest.mark.parametrize("testmodulev2_editable_install", [True])
def test_import(
    testdir: pytest.Testdir, testmodulev2_venv_active: env.VirtualEnv
) -> None:
    """
    Make sure that our plugin works for v2 modules.
    """
    result = testdir.runpytest_inprocess("tests/test_import.py")

    result.assert_outcomes(passed=3)
